import { TokenStandard } from '@metaplex-foundation/mpl-token-metadata';
import { PublicKey, Signer } from '@metaplex-foundation/umi';
import { FreezeTokenPayment, FreezeTokenPaymentArgs } from '../generated';
import { GuardManifest } from '../guards';
/**
 * The freezeTokenPayment guard allows minting frozen NFTs by charging
 * the payer a specific amount of tokens from a certain mint acount.
 * Frozen NFTs cannot be transferred or listed on any marketplaces until thawed.
 *
 * The funds are transferred to a freeze escrow until all NFTs are thaw,
 * at which point, they can be transferred (unlocked) to the configured
 * destination account.
 *
 * @see {@link FreezeTokenPaymentRouteArgs} to learn more about
 * the instructions that can be executed against this guard.
 */
export declare const freezeTokenPaymentGuardManifest: GuardManifest<FreezeTokenPaymentArgs, FreezeTokenPayment, FreezeTokenPaymentMintArgs, FreezeTokenPaymentRouteArgs>;
export declare type FreezeTokenPaymentMintArgs = Omit<FreezeTokenPaymentArgs, 'amount'> & {
    /** The ruleSet of the minted NFT, if any. */
    nftRuleSet?: PublicKey;
};
/**
 * The settings for the freezeTokenPayment guard that should be provided
 * when accessing the guard's special "route" instruction.
 */
export declare type FreezeTokenPaymentRouteArgs = FreezeTokenPaymentRouteArgsInitialize | FreezeTokenPaymentRouteArgsThaw | FreezeTokenPaymentRouteArgsUnlockFunds;
/**
 * The `initialize` path creates the freeze escrow account that will
 * hold the funds until all NFTs are thawed. It must be called before
 * any NFTs can be minted.
 *
 * ```ts
 * route(umi, {
 *   // ...
 *   guard: 'freezeTokenPayment',
 *   routeArgs: {
 *     path: 'initialize',
 *     mint: tokenMint.publicKey,
 *     destinationAta,
 *     period: 15 * 24 * 60 * 60, // 15 days.
 *     candyGuardAuthority,
 *   },
 * });
 * ```
 */
export declare type FreezeTokenPaymentRouteArgsInitialize = Omit<FreezeTokenPaymentArgs, 'amount'> & {
    /** Selects the path to execute in the route instruction. */
    path: 'initialize';
    /** The freeze period in seconds (maximum 30 days). */
    period: number;
    /** The authority of the Candy Guard as a Signer. */
    candyGuardAuthority: Signer;
};
/**
 * The `thaw` path unfreezes one NFT if one of the following conditions are met:
 * - All NFTs have been minted.
 * - The configured period has elapsed (max 30 days).
 * - The Candy Machine account was deleted.
 *
 * Anyone can call this instruction. Since the funds are not transferrable
 * until all NFTs are thawed, it creates an incentive for the treasury to
 * thaw all NFTs as soon as possible.
 *
 * ```ts
 * route(umi, {
 *   // ...
 *   guard: 'freezeTokenPayment',
 *   routeArgs: {
 *     path: 'thaw',
 *     mint: tokenMint.publicKey,
 *     destinationAta,
 *     nftMint,
 *     nftOwner,
 *     nftTokenStandard: candyMachine.tokenStandard,
 *   },
 * });
 * ```
 */
export declare type FreezeTokenPaymentRouteArgsThaw = Omit<FreezeTokenPaymentArgs, 'amount'> & {
    /** Selects the path to execute in the route instruction. */
    path: 'thaw';
    /** The mint address of the NFT to thaw. */
    nftMint: PublicKey;
    /** The owner address of the NFT to thaw. */
    nftOwner: PublicKey;
    /** The token standard of the minted NFT. */
    nftTokenStandard: TokenStandard;
    /** The ruleSet of the minted NFT, if any. */
    nftRuleSet?: PublicKey;
};
/**
 * The `unlockFunds` path transfers all of the escrow funds to the
 * configured destination token address once all NFTs have been thawed.
 *
 * ```ts
 * route(umi, {
 *   // ...
 *   guard: 'freezeTokenPayment',
 *   routeArgs: {
 *     path: 'unlockFunds',
 *     mint: tokenMint.publicKey,
 *     destinationAta,
 *     candyGuardAuthority,
 *   },
 * });
 * ```
 */
export declare type FreezeTokenPaymentRouteArgsUnlockFunds = Omit<FreezeTokenPaymentArgs, 'amount'> & {
    /** Selects the path to execute in the route instruction. */
    path: 'unlockFunds';
    /** The authority of the Candy Guard as a Signer. */
    candyGuardAuthority: Signer;
};
